'<ADbasic Header, Headerversion 001.001>
' Foldings                       = 30,165,233
'<Header End>
'ADBASIC-SYSTEM-FILE--V1.00--0000--0000--INC--GOLD
'
'###############################################################
'#                        Gold-CAN.inc                         #
'#              INCLUDE-file for ADwin-Gold-system             #
'###############################################################
'#                                                             #
'# Contents: Summary of functions and procedures               #
'#           for ADwin-Gold-CAN-Interface.                     #
'#                                                             #
'#  03.02.2004  OL   creation                                  #
'#  14.06.2004  MF                                             #
'#  30.07.2004  JK   ADWGCOM.inc   ->  ADWGCAN.inc             #
'#  14.05.2007  MF   designed fot ADbasic 5                    #
'#  19.02.2009  MF  Transmit_Status                            #
'#  18.03.2009  MF  '''+                                       #
'#  22.07.2010  MF if adwin_system                             #
'#  20.09.2016  MF transmit_status hide removed                #
'#                                                             #
'# Attention:                                                  #
'#   - Do not copy the first source line (containing           #
'#     "'ADBASIC-SYSTEM-FILE...") to user files !              #
'#     Otherwise the Syntax color may be not correct.          #
'#   - This includefile requires ADbasic 4.0 or higher!        #
'#   - arrays which should be transfered to lib-functions must #
'#   - be declared at external DRAM                            #
'#                                                             #
'###############################################################

'<REGION> Declaration
dim CAN_msg[9] as Long         ' array with 9 elements 
' element 1 to 8 for databyte 1 to 8 in CAN-Frame
' element 9 for the number of databytes in CAN-Frame 
import gold_can_lib.li9



#if adwin_system=ADwin_Gold
#else
#error("This include file can only be used with the ADwin Gold")  
#endif
'<ENDREGION>

'<REGION> CAN
'------------------CAN


'***************************************************************
'''Checks if there is a new frame in the selected
'''+message object. If there is a new frame all databytes are read
'''+and then written to the array "can_msg".
'''+The number of dadabytes is written to the element 9 of the array "can_msg".
'''+The return value of the function is the identifier of the frame. 
'''+If there is no new frame in the message box the return value is -1.
'''channel: number of the CAN interface on the CAN module (1..2)
'''msg_no: number of the message object
'''**************************************************************
function Read_Msg(channel,msg_no) as Long
  Read_Msg=lib_read_msg(channel,msg_no,can_msg)
endfunction



'***************************************************************
'''+Checks if there is a new frame in the selected
'''+message object. If there is a new frame all databytes are read
'''+and then written to the array "can_msg".
'''+The number of dadabytes is written to the element 9 of the array "can_msg".
'''+The return value of the function is the identifier of the frame. 
'''+If there is no new frame in the message box the return value is -1.
'''+Checks the consistens of the data and if they are not consistent
'''+the datas will be reread.
'''channel: number of the CAN interface on the CAN module (1..2)
'''msg_no: number of the message object
'''*************************************************************
function Read_Msg_Con(channel,msg_no) as Long
  Read_Msg_Con=lib_read_msg_con(channel,msg_no,can_msg)
endfunction




'*************************************************************
'''Transmits a frame. 
'''+The databytes and the number of databytes have to be written in the
'''+array "can_msg" before this subroutine is activated.
'''+After this subroutine the frame is transmitted as soon as the 
'''+CAN-bus allows it.
'''channel: number of the CAN interface on the CAN module (1..2)
'''msg_no: number of the message object
'''*************************************************************
sub Transmit(channel,msg_no)
  lib_transmit(channel,msg_no,can_msg)
endsub

'---------------------------------------------------------------
''' Checks if a Messagebox is ready to send (The last Message is
'''+ already sent).
'''+ return: 
'''+       0 = ready to send
'''+       1 = not ready to send
''' channel: number of the CAN interface on the CAN module (1..2)
''' msg_no: number of the message object
'''-------------------------------------------------------------
Function Transmit_Status(channel, msg_no) as long 
  Transmit_Status = shift_right(Get_CAN_Reg(channel,(msg_no * 10h) + 1), 5) and 1    ' Control 1register
EndFunction

'***************************************************************
'''Sets a register of the CAN-controller.
'''channel: number of the CAN interface on the CAN module (1..2)
'''register_no: register which should be set
'''value: value which should be written to the register.
'''**************************************************************
sub Set_CAN_Reg(channel,register_no, value)   
  dim adr1 as Long  
  adr1 = 20400000h or register_no  or (500h + ((channel -1) * 100h))
  poke(adr1 ,value)                      'Writing adress to the CAN-controller
endsub


'***************************************************************
'''Reads a register of the CAN-controller.
'''+return value: value of the register
'''channel: number of the CAN interface on the CAN module (1..2) 
'''register_no: register to read
'''*************************************************************
Function Get_CAN_Reg(channel,register_no) as Long
  dim adr1 as Long  
  adr1 = 20400000h or register_no  or (500h + ((channel -1) * 100h))
  get_can_reg = peek(adr1)
endfunction

  
'***************************************************************
'''Reads the register where the bus error pin of the tranceiver
'''+is stored.
'''+The register is set when the a bus error occurs and is reset by 
'''+reading the register.
'''+return value: 
'''+0 = no error
'''+1 = error
'''+Attention: This command is only for low-speed CAN
'''channel: number of the CAN interface on the CAN module (1..2)
'''**************************************************************  
Function Get_Line_Status(channel) as Long
  dim value as Long
  Get_Line_Status =(peek(20400421H+(channel-1)*2)) AND 1
endfunction


'***************************************************************
'''Enables the interrupt of a message object.
'''channel: number of the CAN interface on the CAN module (1..2)
'''+             (also necessary if the module provides only one channel)
'''msg_no: number of the message object
'''*************************************************************
sub En_CAN_Interrupt(channel,msg_no)
  set_can_reg(channel, msg_no * 16, 0ebh)   'Enable interrupt of the message object.
  set_can_reg(channel, 0, 02h)              'Enable global interrupt.
  poke (20400425h,(peek(20400425h) or channel))
endsub
'<ENDREGION>

'<REGION> SSI
'**************************************************************
''' Reads SSI encoder value 32 Bit. 
''' channel: number of the counter (1..4)
'''************************************************************
FUNCTION SSI_Read(channel) AS Long
  SSI_READ = PEEK(2040043Fh + channel)      'read 32 Bit-value
ENDFUNCTION


'**************************************************************
''' Sets the number of databits for the SSI Interface.
''' channel: number of the counter (1..4)
''' value: number of bits
'''************************************************************
SUB SSI_Set_Bits(channel,value)
  poke((20400443h + channel) , value)           
ENDSUB


'**************************************************************
''' Sets frequency of the SSI clock.
''' channel: number of the counter (1..4)
''' prescale: frequency = 10MHZ / value
'''************************************************************
SUB SSI_Set_Clock(channel,prescale)
  poke((20400447h + channel), prescale)           
ENDSUB


'**************************************************************
''' Starts the SSI interface.
''' pattern: Bitcombination each bit corresponts to one counter
'''************************************************************
SUB SSI_Start(pattern)
  Dim value2 as Long
  value2=(pattern and 0fH) or peek(2040044ch)
  poke(2040044ch, value2)
ENDSUB


'*************************************************************
''' Sets the SSI mode.
''' pattern: Bitcombination each bit corresponts to one counter
'''+         0 = single start mode
'''+         1 = continuous start mode
'''***********************************************************
sub SSI_Mode(pattern)
  dim value3 as Long
  value3= shift_left(pattern,4) AND 0F0H
  poke(2040044ch, ((peek(2040044ch) AND 0FH)OR value3))
endsub


'*************************************************************
''' Reads the SSI status.
'''+ return value 
'''+   0 = SSI value compleated
'''+   1 = SSI transfer running
''' channel: number of the counter (1..4)
'''***********************************************************
FUNCTION SSI_Status(channel) AS Long
  dim value2 as Long
  value2 = peek(2040044ch)
  ssi_status=shift_right(value2,(channel-1)) AND 01H
ENDFUNCTION
'<ENDREGION>

'<REGION> RSxxx

'***********************************************
''' Sets a register of the serial controller.
''' register_no: register on the serial controller
''' value: value for the selected register
'''*********************************************
sub Set_RS(register_no,value)
  poke (20400400h + register_no, value) 
endsub


'***********************************************
''' Reads the value of a register of the serial controller.
'''register_no: register on the serial controller
'''*********************************************
FUNCTION Get_RS(register_no) AS Long
  Get_RS = peek(20400400h + register_no)
ENDFUNCTION


'***************************************************************
''''Resets the serial interface module.
'''*************************************************************
sub RS_Reset()
  poke(20400424h, 1)            ' reset aktiv
  poke(20400424h, 0)            ' reset inaktiv
  poke(2040042Ch, 0)            'RS 485 Control
  poke(2040042Dh, 0)            'RS 485 Control
endsub


'***************************************************************
'''Only for RS485.
'''+Selects whether the interface is sending data or is listening on the bus.
'''channel: number of the serial interface (1..2)
'''dir:
'''+           0 = Listening
'''+           1 = Sending Data
'''+           2 = Sending Data + Listening
'''*************************************************************
sub RS485_Send(channel,dir)
  poke(2040042Bh+channel,((dir+1)AND 3))   'setting the RTS-Pin     
endsub

'<ENDREGION>

