'<ADbasic Header, Headerversion 001.001>
' Foldings                       = 33,37,118,174,265
'<Header End>
'ADBASIC-SYSTEM-FILE--V1.00--0000--0000--INC--PRO
'
'###############################################################
'#                        ADwpDA.inc                           #
'#              INCLUDE-file for ADwin-Pro-system              #
'###############################################################
'#                                                             #
'# Contents: Definitions of functions and procedures           #
'#           to access D/A-modules.                            #
'#                                                             #
'#  25.10.1996  OL   creation                                  #
'#  10.06.1999  OL   redesign & comments in english            #
'#  03.12.2001  JK   1st line : ADBASIC-SYSTEM_FILE...         #
'#  02.09.2002  MG   T10                                       #
'#  05.10.2002  MF   T10                                       #
'#  27.01.2004 MF/TN new commands for memory option            #
'#                   (e.g. Pro-Aout-4/16-M2)                   #
'#  03.02.2005  MF    NOT T11                                  #
'#  14.04.2005  MF   ERROR WRONG PROCESSOR                     #
'#  23.01.2006  MF   WRITEDAC32                                #
'#  16.03.2007  MF   designed for ADbasic 5                    #
'#  23.03.2009  MF   '''+                                      #
'#  10.08.2010  MF   syntax_highlight AD,DA,DIO,EXT            #
'#  23.02.2016  mf   bugfix writedac32                         #
'# Attention:                                                  #
'#   - Do not copy the first source line (containing           #
'#     "'ADBASIC-SYSTEM-FILE...") to user files !              #
'#     Otherwise the Syntax color may be not correct.          #
'#   - This includefile requires ADbasic 4.0 or higher!        #
'#                                                             #
'###############################################################

#IF PROZESSOR=T11 THEN
#error("This include file can only be used with the processor T2, T4, T5, T8, T9 or T10.")
#ELSE

'<REGION> Declaration
'---------------------------------------------------------------
' Base-address for the different types of Pro-modules.
'---------------------------------------------------------------

'''system constant which can be used by:
'''SetLED,CheckLED
'''EventEnable,EventSource
'''SyncEnable,SyncStat
#DEFINE DIO 000h  'DIO - MASK 


'''system constant which can be used by:
'''SetLED,CheckLED
'''EventEnable,EventSource
'''SyncEnable,SyncStat
#DEFINE AD  040h  'A/D - MASK 


'''system constant which can be used by:
'''SetLED,CheckLED
'''EventEnable,EventSource
'''SyncEnable,SyncStat
#DEFINE DA  080h  'D/A - MASK 


'''system constant which can be used by:
'''SetLED,CheckLED
#DEFINE CPU 0A0h  'CPU - MASK 

'''system constant which can be used by:
'''SetLED,CheckLED
'''EventEnable,EventSource
'''SyncEnable,SyncStat
#DEFINE EXT 0C0h  'EXT - MASK 


'---------------------------------------------------------------
' Address-table to access the hardware of the Pro-system,
' depending on the CPU in use (ADSP or Transputer).
'---------------------------------------------------------------
#IF PROZESSOR = T9 THEN
#define DL         20400000h ''<HIDE>
#define DH         20400010h ''<HIDE>
#define DLX        20400020h ''<HIDE>
#define DHX        20400030h ''<HIDE>
#define ADWR       20400060h ''<HIDE>
#define ADRD       20400070h ''<HIDE>
#define CPU_EVENT  204000F0h ''<HIDE>
#define WATCHDOG   204001D0h ''<HIDE>
#define WD_TIMER   204001E0h ''<HIDE>
#define CPU_LED    204001F0h ''<HIDE>
#define D32        20400180h ''<HIDE>
#ELSE
#IF PROZESSOR = T10 THEN
#define DL         20800000h ''<HIDE>
#define DH         20800010h ''<HIDE>
#define DLX        20800020h ''<HIDE>
#define DHX        20800030h ''<HIDE>
#define ADWR       20800060h ''<HIDE>
#define ADRD       20800070h ''<HIDE>
#define CPU_EVENT  208000F0h ''<HIDE>
#define WATCHDOG   208001D0h ''<HIDE>
#define WD_TIMER   208001E0h ''<HIDE>
#define CPU_LED    208001F0h ''<HIDE>
#define D32        20800180h ''<HIDE>
#ELSE
#define DL               00h ''<HIDE>
#define DH               10h ''<HIDE>
#define DLX              20h ''<HIDE>
#define DHX              30h ''<HIDE>
#define ADWR             60h ''<HIDE>
#define ADRD             70h ''<HIDE>
#define CPU_EVENT       0F0h ''<HIDE>
#define WATCHDOG        1D0h ''<HIDE>
#define WD_TIMER        1E0h ''<HIDE>
#define CPU_LED         1F0h ''<HIDE>
#ENDIF
#ENDIF
'<ENDREGION>

'<REGION> Helper functions
'---------------------------------------------------------------
' Procedure which adds five NOP-instructions after a REG.-WRITE
' to compensate delaytimes.
'---------------------------------------------------------------
SUB poke_nop(adress,value) ''<HIDE>
#IF PROZESSOR = T9 THEN
  poke(adress,value)
  nops(5)   
#ELSE
#IF PROZESSOR = T10 THEN
  poke(adress,value)
  nops(10)
#ELSE
  poke(adress,value)
#ENDIF
#ENDIF
ENDSUB


'---------------------------------------------------------------
' Procedure which adds one additional NOP-instruction only for
' the ADSP processor, because only neccessary there.
'---------------------------------------------------------------
SUB NOP_T9() ''<HIDE>
#IF PROZESSOR = T9 THEN
  NOP
#ENDIF
#IF PROZESSOR = T10 THEN
  NOP
  NOP
#ENDIF
ENDSUB


'---------------------------------------------------------------
' IF a low-priority process access a Pro-module it must not be
' interrupted by any other process !!!
'---------------------------------------------------------------
SUB SUB_BEGIN_CRITICAL_SECTION() ''<HIDE>
#IF PROZESSOR = ADSP THEN
#IF PRIORITAET = LOW
  BEGIN_CRITICAL_SECTION
#ENDIF
#ENDIF
ENDSUB

SUB SUB_END_CRITICAL_SECTION()''<HIDE>
#IF PROZESSOR = ADSP THEN
#IF PRIORITAET = LOW
  END_CRITICAL_SECTION
#ENDIF
#ENDIF
ENDSUB
'<ENDREGION>

'<REGION> Analog output
'***********************' Convert the digital value to an analog output
'''Outputs an (analog) voltage on the channel of the specified module, that corresponds to the indicated digital value
'''module: module address (0..255)
'''dac_no: channel number of analog output
'''value: output value (0..65535)
'''***********************
SUB DAC(module,dac_no,value)
  DIM addr, dac_n,out_value AS INTEGER
  out_value = value
  IF (out_value < 0) THEN out_value = 0
  IF (out_value > 0FFFFh) THEN out_value = 0FFFFh
  PRECALC
  dac_n = dac_no-1
  PRECALC
  addr = (dac_n OR 080h) OR (SHIFT_LEFT(module,8))
  SUB_BEGIN_CRITICAL_SECTION
  POKE(ADWR,addr)
  NOP_T9
  Poke_NOP(DLX,out_value)
  SUB_END_CRITICAL_SECTION
  PRECALC
  addr = 090h OR (SHIFT_LEFT(module,8))
  SUB_BEGIN_CRITICAL_SECTION
  POKE(ADWR,addr)
  NOP_T9
  Poke_NOP(DLX,0)
  SUB_END_CRITICAL_SECTION
ENDSUB


'***************************************************************
'''Sets DAC-register to desired value.
'''module: module address (0..255)
'''dac_no: channel number of analog output
'''value: output value (0..65535)
'''*************************************************************
SUB WriteDAC(module,dac_no,value)
  DIM addr, dac_n, out_value AS INTEGER
  out_value = value
  IF (out_value < 0) THEN out_value = 0
  IF (out_value > 0FFFFh) THEN out_value = 0FFFFh
  PRECALC
  dac_n = dac_no-1
  PRECALC
  addr = (dac_n OR 080h) OR (SHIFT_LEFT(module,8))
  SUB_BEGIN_CRITICAL_SECTION
  POKE(ADWR,addr)
  NOP_T9
  Poke_NOP(DLX,out_value)
  SUB_END_CRITICAL_SECTION
ENDSUB

' ***************************************************************
'''Sets DAC-register to desired value. 
'''+ only AOUT Rev B or C
'''module: module address (0..255)
'''dac_no: 
'''+         0 = Channel 1 and 2
'''+         1 = Channel 3 and 4
'''+         2 = Channel 5 and 6
'''+         3 = Channel 7 and 8
'''value: Value to output (32 Bit)
'''*************************************************************
SUB WriteDAC32(module,dac_no,value)
  DIM addr AS INTEGER
  addr = ((dac_no) OR 0A0h) OR (SHIFT_LEFT(module,8))
  SUB_BEGIN_CRITICAL_SECTION
  POKE(ADWR,addr)
  NOP_T9
  Poke_NOP(d32,value)
  SUB_END_CRITICAL_SECTION
ENDSUB


'***************************************************************
'''Simultaneously starts conversion of all DAC on this module.
'''module: module address (0..255)
'''*************************************************************
SUB Start_DAC(module)
  DIM addr, dac_n AS INTEGER
  PRECALC
  addr = 090h OR (SHIFT_LEFT(module,8))
  SUB_BEGIN_CRITICAL_SECTION
  POKE(ADWR,addr)
  NOP_T9
  Poke_NOP(DLX,0)
  SUB_END_CRITICAL_SECTION
ENDSUB
'<ENDREGION>

'<REGION> Frequency generator

'****************************************************************
' NEW COMMANDS FOR DAC Module with MEM option
'****************************************************************

'****************************************************************
'''Sets the mode of the dac module.
'''+This sub is only for ADSP processors and AOUT with MEM option
'''module: module address (0..255)
'''mode:
'''+      0  normal mode 
'''+      1  frequency generator mode
'''**************************************************************
SUB FG_Mode(module,mode)   
  DIM adr as integer
  adr = (23 OR 080h) OR (SHIFT_LEFT(module,8))
  
  SUB_BEGIN_CRITICAL_SECTION
  poke(ADWR , adr )
  nop_t9
  poke_nop( DLX , 0)
  SUB_END_CRITICAL_SECTION

  SUB_BEGIN_CRITICAL_SECTION
  poke(ADWR , adr )
  nop_t9
  poke_nop( DLX , mode)
  SUB_END_CRITICAL_SECTION
ENDSUB

'****************************************************************
'''Sets the delay between two steps.
'''+This sub is only for ADSP processors and AOUT with MEM option
'''module: module address (0..255)
'''dac_no: channel number of analog output
'''scale: F_output=80 MHz/scale
'''**************************************************************
SUB FG_Delay(module, dac_no, scale)                                    '                                     
  DIM adr as integer
  adr = (10 OR 080h) OR (SHIFT_LEFT(module,8))
  SUB_BEGIN_CRITICAL_SECTION
  poke( ADWR, adr)
  nop_t9
  poke_nop( D32 ,(scale-2) or shift_left((dac_no)-1,29))  
  SUB_END_CRITICAL_SECTION
EndSub  


'****************************************************************
'''Sets the buffer of each channel.
'''+This sub is only for ADSP processors and AOUT with MEM option
'''module: module address (0..255)
'''dac_no: channel number of analog output
'''startadr: start_adr of memory
'''memsize: size of memory for this channel
'''**************************************************************
SUB FG_Def(module, dac_no, startadr, memsize)  
  DIM adr as integer
  
  adr = (8 OR 080h) OR (SHIFT_LEFT(module,8))
  
  SUB_BEGIN_CRITICAL_SECTION
  poke( ADWR, adr)
  nop_t9
  poke_nop( D32, (startadr) OR shift_left((dac_no-1),29))  
  SUB_END_CRITICAL_SECTION
 
  inc adr

  SUB_BEGIN_CRITICAL_SECTION
  poke( ADWR, adr)
  nop_t9
  poke_nop( D32, (memsize) OR shift_left((dac_no-1),29))
  SUB_END_CRITICAL_SECTION
ENDSUB

'**************************************************************
'''Transfers the output data from a array to the memory of the dac module.
'''+This sub is only for ADSP processors and AOUT with MEM option
'''module: module address (0..255)
'''mem_start_adr: start_adr of memory
'''count: number of values (only even-numbered)
'''array[]: source array 
'''array_startindex: startindex in the source array
'''************************************************************
SUB FG_Write(module,mem_start_adr,count,array[],array_startindex)

  DIM ADR AS Long
  DIM value AS Long
  DIM i AS Long
  dim lenint as Long
  dim pointer as Long
  dim hw as Long
  dim lw as Long

  ADR=(18 OR 080h) OR (SHIFT_LEFT(module,8))

  SUB_BEGIN_CRITICAL_SECTION
  poke( ADWR, adr)
  nop_t9
  poke_nop(d32,((mem_start_adr) or (080000000H)) )
  SUB_END_CRITICAL_SECTION

  pointer=array_startindex
  inc adr 
 
  lenint=count 
 
  if (LENINT>1) then
    for i=1 to LENINT step 2   
   
      lw=array[pointer] AND 0FFFFH
      inc pointer
      hw=shift_left(array[pointer],16)
      inc pointer
      value=lw or hw
   
      SUB_BEGIN_CRITICAL_SECTION 
      poke(ADWR,adr)
      nop_t9
      poke_nop(d32,value)
      SUB_END_CRITICAL_SECTION
    NEXT
  endif
 
ENDSUB

'**************************************************************
'''Transfers the waveform data from memory to an array.
'''+This sub is only for ADSP processors and AOUT with MEM option
'''module: module address (0..255)
'''mem_start_adr:  start_adr of memory
'''count: number of values (only even-numbered)
'''array[]: destination array 
'''array_startindex: startindex in the destination array
'''************************************************************
SUB FG_Read(module,mem_start_adr,count,array[],array_startindex)

  DIM ADR AS Long
  DIM value AS Long
  DIM i AS Long
  dim lenint as Long
  dim pointer as Long
  dim hw as Long
  dim lw as Long

  ADR=(18 OR 080h) OR (SHIFT_LEFT(module,8))

  SUB_BEGIN_CRITICAL_SECTION
  poke( ADWR, adr)
  nop_t9
  poke_nop( d32 , (mem_start_adr))
  SUB_END_CRITICAL_SECTION

  pointer=array_startindex
  inc adr 
 
  sleep(6)
 
  lenint=count 
 
  if (LENINT>1) then
    for i=1 to LENINT step 2   
      SUB_BEGIN_CRITICAL_SECTION 
      poke_nop(ADRD,adr)
      value=peek(d32)
      SUB_END_CRITICAL_SECTION
   
     
      array[pointer]=value AND 0FFFFH
      inc pointer
      array[pointer]=shift_right(value,16)
      inc pointer
    NEXT
  endif
 
ENDSUB


'****************************************************************
'''Reads the actual output-position of channel 1-8 in the RAM.
'''+This sub is only for ADSP processors and AOUT with MEM option
'''module: module address (0..255)
'''dac_no: channel number of analog output
'''**************************************************************
FUNCTION FG_Read_Index(module, dac_no) AS Long
  DIM adr as integer
  adr = ((7+(dac_no))  OR 080h) OR (SHIFT_LEFT(module,8))
  SUB_BEGIN_CRITICAL_SECTION
  poke_nop(ADRD, adr )
  FG_READ_INDEX= peek(d32) and 01FFFFFh  
  SUB_END_CRITICAL_SECTION
  nop_t9
ENDFUNCTION

'****************************************************************
'''Starts or Stops frequency generator.
'''+ This sub is only for ADSP processors and AOUT with MEM option  
'''module: module address (0..255)
'''pattern: bitcombination; each bit corresponds with an output
'''run_mode: 
'''+action=0 starts output of Dacs immediately 
'''+action=1 stops output of Dacs immediately 
'''+action=2 stops output of Dacs after the periode     
'''***************************************************************
SUB FG_Control(module,pattern,run_mode)                                     
  DIM adr as integer
  adr = ((20 OR 080h) OR (SHIFT_LEFT(module,8)))+run_mode
  SUB_BEGIN_CRITICAL_SECTION
  poke(ADWR , adr )
  nop_t9
  poke_nop( DLX , (pattern) AND 0FFFFH )
  SUB_END_CRITICAL_SECTION
ENDSUB

'****************************************************************
'''Reads status.
'''+value : bitcombination ; each bit corresponds with an output
'''+value=1 generator of the channel is running
'''+value=0 generator of the channel is stopped
'''+This sub is only for ADSP processors and AOUT with MEM option
'''module: module address (0..255)
'''**************************************************************
FUNCTION FG_Status(module) AS Long
  DIM adr as integer
  adr = (20 OR 080h) OR (SHIFT_LEFT(module,8))
  SUB_BEGIN_CRITICAL_SECTION
  poke_nop(ADRD, adr )
  FG_STATUS= peek(DLX) and 0Fh  
  SUB_END_CRITICAL_SECTION
  nop_t9
ENDFUNCTION
'<ENDREGION>

#ENDIF
