'<ADbasic Header, Headerversion 001.001>
' Foldings                       = 59,88,120
'<Header End>
'ADBASIC-SYSTEM-FILE--V1.00--0000--0000--INC--ALL
'################################################################
'# The following functions are directly implemented in the      #
'# ADbasic compilers :                                          #
'#   AbsF, AbsI, Dec, Inc, Max_Float, Max_Long,                 #
'#   Min_Float, Min_Long, Round, ^ (Power), Exp, LN, Log, Sqrt, #
'#   ArcCos, ArcSin, ArcTan, ArcTan2, Cos, Sin, Tan             #
'# For information on these, please use the ADbasic help.       #
'################################################################
'#                           MATH.inc                           #
'#              INCLUDE-file for all ADwin-Systems              #
'################################################################
'#                                                              #
'# Contents: Mathematical FUNCTIONs for processors T9, T10, T11 #
'#                                                              #
'#  12.11.2009  TN   creation (Beta)                            #
'#  02.02.2010  TN   documentation of Mod                       #
'#  25.03.2010  JK   comments  ;  released for ADwin-CD         #
'#  26.01.2011  TN   div_by_10                                  #
'#  19.09.2014  TN   T12                                        #
'#  27.10.2014  JK   div_by_10 for T12                          #
'#  19.05.2017  MF   T121                                       #
'#  23.01.2024  PJ   Rand(), trigonometric hyperbolic functions #
'#                                  (Sinh(), Cosh(), ...)       #
'#  26.01.2024  JK   just some comments                         #
'#                                                              #
'# Attention:                                                   #
'#   - Do not copy the first source line (containing            #
'#     "'ADBASIC-SYSTEM-FILE...") to user files !               #
'#     Otherwise the Syntax color may be not correct.           #
'#                                                              #
'################################################################

#If Processor = T9 Then
Import Math.Li9
#EndIf
#If Processor = T10 Then
Import Math.LiA
#EndIf
#If Processor = T11 Then
Import Math.LiB
#EndIf
#If Processor = T12 Then
Import Math.LiC
#EndIf
#If Processor = T121 Then
Import Math.1.LiC
#EndIf



' Divides a value by 10. Stores the quotient and the remainder in
' the output parameters.
' see division_strings.doc for a description of the algorithm.
' Time consumption (in cycles), if all parameters are PAR_<x>'s:
' T9   |  T10  |  T11 (Opt.-Level 2)  | T11 (Opt.-Level 0 and 1
' -----+-------+----------------------+--------------------------
' 39 ? | 21 ?  |                      | 18
Sub Div_by_10 (dividend, quotient, remainder)
#If Processor = T11, T12, T121
  Dim temp As Long
  temp = dividend
  quotient = temp * 0.1
  remainder = temp - quotient * 10
#Else
  Dim temp As Long 
  Dim sign_of_result As Long 
  Dim temp_dividend As Long
  
    
  If (dividend < 0) Then 
    temp_dividend = -dividend
    sign_of_result = -1
  Else
    temp_dividend = dividend
    sign_of_result = 1
  EndIf
    
  temp = Shift_Right ((temp_dividend) * 019999999H + Shift_Right (temp_dividend, 1) + Shift_Right (temp_dividend, 3), 28)
  remainder = Shift_Right (10 * (temp + 1), 4)
  quotient = Shift_Right ((temp_dividend) - remainder, 1) * 0CCCCCCCDH * sign_of_result
#EndIf
EndSub


#If Processor = T12, T121

'<Region> "Random Functions"
 
' srand seeds the random function with a 32 bit value depending on this seed
' the values generated represent a deterministic sequence. If the random function is not seeded
' it will use 1 as its default seed. It is recomended to seed the random function
' with the read_timer() value to generate different sequences.
Sub SRand(seed)
  lib_srand(seed)
EndSub

' returns the next value in the seeded sequence
' 0 - 2147483647
Function Rand() As Long
  Rand = lib_rand()
EndFunction

' low (inclusive) - high (exclusive) 
Function Rand_Range(low, high) As Long
  Rand_Range = lib_rand_range(low, high)
EndFunction

' 0.0 - 1.0
Function RandF() As Float
  RandF = lib_randf()
EndFunction

' low (inclusive) - high (inclusive) 
Function Rand_RangeF(low, high) As Float
  Rand_RangeF = lib_rand_rangef(low, high)
EndFunction
'<EndRegion>

'<Region> "Hyperbolic Functions"

' https://en.wikipedia.org/wiki/Hyperbolic_functions

Function SinH(angle) As Float
  SinH = lib_sinh(angle)
EndFunction

Function ASinH(angle) As Float
  ASinH = lib_asinh(angle)
EndFunction

Function CosH(angle) As Float
  CosH = lib_cosh(angle)
EndFunction

Function ACosH(angle) As Float
  ACosH = lib_acosh(angle)
EndFunction

Function TanH(angle) As Float
  TanH = lib_tanh(angle)
EndFunction

Function ATanH(angle) As Float
  ATanH = lib_atanh(angle)
EndFunction
'<EndRegion>


' The normal Mod() function returns Mod(x, 0) = x
' But the C operator works like this: x % 0 = 0
' To keep compatibility the normal Mod() function won't be changed.
' But to have an equivalent function to C this function is the alternative.
' This function is currently deliberately undocumented! 
Function Mod_C(x_param, y_param) As Long
  Mod_C = lib_mod(x_param, y_param)
EndFunction

Function Mod64_C(x_param, y_param) as int64
  Mod64_C = lib_mod64(x_param, y_param)
EndFunction

' returns sqrt(side1 * side1 + side2 * side2)
' for calculating the length of a Vector or
' the hypotenuse of a right triangle 
' optimised and a bit faster than calculating it
' manually.
Function Hypot(side1, side2) As Float
  Hypot = lib_hypot(side1, side2)
EndFunction

#EndIf  ' T12 , T12.1



'<Region> "Library Functions"


'Lib_Function Mod(ByVal x_param As Long,ByVal y_param As Long) As Long
'#####################################################################
'#                                                                   #
'#                            r = x Mod y                            #
'#                                                                   #
'#   The modulus determines the signed remainder of x divided by y   #
'#              for nonzero y, and returns x otherwise.              #
'# The function uses the definition of Wirth: the return value of    #
'# the division will be truncated. Therefore Mod(-4,3) returns -1.   #
'#                                                                   #
'# The function Mod() expects LONGs as parameters                    #
'#                                                                   #
'# Known issues:  none                                               #
'#####################################################################


'<EndRegion>

