#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>
#include <libgen.h>
#include <ctype.h>
#include <limits.h>

#include "adwin-macros.h"
#include "linux/adwin.h"
#include "libadwin.h"
#include "libadwin/adctl.h"
#include "libadwin/bits.h"
#include "libadwin/errno.h"
#include "libadwin/types.h"
#include "configfile.h"
#include "retry_count.h"
#include "libadwin-macros.h"
#include "getDataSH.h"
#include "ADEthernet.h"

#include <sys/un.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/time.h>
#define USE_FOR_LIBADWIN 1
// todo:
#include "ADEthernet.h"       /* Prototypes for Ethernet functions (NET_*) */

#include "data_convert.h"

#if defined(__FreeBSD__)
    #include <sys/syslimits.h>
#endif

const int32_t g_Version = 2000015;

int NAN_test = 1; // 1: check float values, default

void Set_DeviceNo(int32_t device_no)
{
    adwin_dev_t Dev;
    int fd;
    DeviceNumber dn;
    DeviceNumber NewDn;
    char server_name[INET_ADDRSTRLEN];

    PRINTDEBUG(DBG_GENERIC_DRV, ("Set_DeviceNo(%d)\n", device_no));

    // check if the given device number is valid
    if ((device_no < 1) || (device_no > 0xfff)) {
        adwin_errno = k_this_link_not_allowed;
        return;
    }

    // copy CURRENTDEVICE to dn
    memcpy(&dn, &CURRENTDEVICE, sizeof(DeviceNumber));
    // if CURRENTDEVICE was already device_no, the number did not change

    if (dn.DeviceNo == device_no) {
        //PRINTDEBUG(DBG_GENERIC_DRV, ("No change in Set_DeviceNo()\n"));
        return;
    }

    if ((fd = SETMUTEX (device_no, &Dev)) >= 0) {
        inet_ntop(AF_INET, (const void *)&Dev.dev_net_server.sin_addr,
                  server_name, INET_ADDRSTRLEN); // should not fail...

        if (Dev.dev_type == DC_ETHERNET)
            if (dn.DeviceType == DC_ETHERNET)
                reConnect(server_name,
                          Dev.dev_net_host_password,
                          Dev.dev_net_host_link,
                          ntohs (Dev.dev_net_server.sin_port),
                          Dev.dev_net_timeout);
            else
                openConnect(server_name,
                            Dev.dev_net_host_password,
                            Dev.dev_net_host_link,
                            ntohs (Dev.dev_net_server.sin_port),
                            Dev.dev_net_timeout);
        else if (dn.DeviceType == DC_ETHERNET)
            closeConnect();

        NewDn.DeviceNo = device_no;
        NewDn.DeviceType = Dev.dev_type;
        memcpy(&CURRENTDEVICE, &NewDn, sizeof(DeviceNumber));

        CLEARMUTEX(&Dev, fd);
        adwin_errno = 0;
        return;
    }

    if (adwin_errno == k_this_link_is_unknown)
        return;
    adwin_errno = k_unknown_device;
}

void SanitizeFloatingPointValues(int flag) {
    PRINTDEBUG(DBG_GENERIC_DRV, ("SanitizeFloatingPointValues(%d)\n", flag));
    if (flag == 1)
        NAN_test = 1;
    else
        NAN_test = 0;
}

/*****************************************************************************/

void Boot(const char *filename)
{
    long res;

    res = adwin_boot(filename, 100000, CURRENTDEVICE.DeviceNo);

    if ((res > 1000) && (res != 100000) && (res != 8000)) {
        adwin_boot(filename, res, CURRENTDEVICE.DeviceNo);
    }
}

long adwin_boot(const char *btl, long iboardsize, int id)
{
    adwin_dev_t dev;
    int fd;
    struct stat s;
    char share[PATH_MAX];

    if (stat(btl, &s) < 0) {
        if (! getAdwindir(share)) {
            adwin_errno = -errno;
            if (adwin_errno == -ENOENT) adwin_errno = k_File_not_found;
            return -1;
        }

        strcat(share, "/share/btl/");
        strcat(share, basename((char *)btl));

        if (stat(share, &s) < 0) {
            adwin_errno = -errno;
            if (adwin_errno == -ENOENT) adwin_errno = k_File_not_found;
            return -1;
        } else {
            btl = share;
        }
    }

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Boot(btl);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/
int32_t Test_Version()
{
    return adwin_test_version(CURRENTDEVICE.DeviceNo);
}

int adwin_test_version(int id)
{
    adwin_dev_t dev;
    int fd, tmp = 3;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return tmp;

    tmp = Generic_Driver_Test_Version();
    CLEARMUTEX(&dev, fd);

    return tmp;
}

/*****************************************************************************/

processor_type_t Processor_Type()
{
    return adwin_processor_type(CURRENTDEVICE.DeviceNo);
}

processor_type_t adwin_processor_type(int id)
{
    adwin_dev_t dev;
    int fd;
    processor_type_t dest = -1;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    dest = Generic_Driver_Processor_Type();
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    return adwin_errno ? -1 : dest;
}

/*****************************************************************************/

int32_t Workload(int32_t priority)
{
    return adwin_workload(priority, CURRENTDEVICE.DeviceNo);
}

int adwin_workload(int priority, int id)
{
    int err, value;
    value = __adwin_workload(priority, &err, id);

    return (err == -1) ? err : value;
}

int __adwin_workload(int priority, int *err, int id)
{
    adwin_dev_t dev;
    int fd, tmp = -1;
    int32_t dest = -1;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return tmp;

    dest = Generic_Driver_Workload(priority);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    if (err)
        *err = adwin_errno;

    return dest;
}

/*****************************************************************************/

int32_t Free_Mem(mem_spec_t mem_spec)
{
    return adwin_mem(mem_spec, CURRENTDEVICE.DeviceNo);
}

long adwin_mem(mem_spec_t mem_spec, int id)
{
    int err, memsize;
    memsize = __adwin_mem(mem_spec, &err, id);
    return (err == -1 ? err : memsize);
}

long __adwin_mem(mem_spec_t mem_spec, int *err, int id)
{
    adwin_dev_t dev;
    int fd;
    int32_t dest = -1;
    processor_type_t processor;

    /* first, we need to determine the processor type */
    processor = adwin_processor_type(id);
    /* todo: if this fails, we have two timeouts here... */

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    dest = Generic_Driver_Free_Mem(mem_spec);
    adwin_errno = Generic_Driver_Get_Last_Error();

    switch (mem_spec) {
        case MEM_PM_LOCAL:
            if ((processor == PROCESSOR_T9) || (processor == PROCESSOR_T10))
                dest *= 6;
            else
                dest *= 4;
            break;
        case MEM_ALL:
        case MEM_EM_LOCAL:
        case MEM_DM_LOCAL:
        case MEM_DRAM_EXTERN:
            dest *= 4;
            break;
        case MEM_UM:
        case MEM_CM:
            break;
        default:
            adwin_errno = -EINVAL;
            dest = -1;
    }
    CLEARMUTEX(&dev, fd);

    if (err)
        *err = adwin_errno;

    return dest;
}

/*****************************************************************************/

void Load_Process(const char *Filename)
{
    adwin_process_load(Filename, CURRENTDEVICE.DeviceNo);
}

int adwin_process_load(const char *filename, int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Load_Process(filename);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

void Start_Process(int32_t process_no)
{
    adwin_process_start(process_no, CURRENTDEVICE.DeviceNo);
}

int adwin_process_start(int process_no, int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Start_Process(process_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

void Stop_Process(int32_t ProcessNo)
{
    adwin_process_stop(ProcessNo, CURRENTDEVICE.DeviceNo);
}

int adwin_process_stop(int processID, int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Stop_Process(processID);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

void Clear_Process(int32_t ProcessNo)
{
    adwin_process_clear(ProcessNo, CURRENTDEVICE.DeviceNo);
}

int adwin_process_clear(int processID, int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return 1;

    Generic_Driver_Clear_Process(processID);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? 1 : 0;
}

/*****************************************************************************/

int32_t Process_Status(int32_t ProcessNo)
{
    return adwin_process_status(ProcessNo, CURRENTDEVICE.DeviceNo);
}

int adwin_process_status(int ProcessNo, int id)
{
    return adwin_get_par(ProcessNo - 100, id);
}
/*****************************************************************************/

void Set_Processdelay(int32_t ProcessNo, int32_t Value)
{
    adwin_set_processdelay(ProcessNo, Value, CURRENTDEVICE.DeviceNo);
}

int adwin_set_processdelay(int process, int delay, int id)
{
    return adwin_set_par(process - 90, delay, id);
}

/*****************************************************************************/

int32_t Get_Processdelay(int32_t ProcessNo)
{
    return adwin_get_processdelay(ProcessNo, CURRENTDEVICE.DeviceNo);
}

int adwin_get_processdelay(int process, int id)
{
    return adwin_get_par(process - 90, id);
}

int __adwin_get_processdelay(int process, int *err, int id)
{
    return __adwin_get_par(process - 90, err, id);
}

/*****************************************************************************/
void Set_Par(int32_t Index, int32_t Value)
{
    adwin_set_par(Index, Value, CURRENTDEVICE.DeviceNo);
}

int adwin_set_par(int parID, int value, int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Set_Par(parID, value);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

int32_t Get_Par(int32_t Index)
{
    return adwin_get_par(Index, CURRENTDEVICE.DeviceNo);
}

int adwin_get_par(int parID, int id)
{
    int err, value;
    value = __adwin_get_par(parID, &err, id);

    return ( err == -1) ? err : value;
}

/* Linux-only */
int __adwin_get_par(int parID, int *err, int id)
{
    adwin_dev_t dev;
    int fd;
    int32_t dest = -1;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    dest = Generic_Driver_Get_Par(parID);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    if (err)
        *err = adwin_errno;

    return dest;
}

/*****************************************************************************/
void Get_Par_Block(int32_t dest[], int start_index, int count)
{
    adwin_get_par_block(start_index, count, dest, CURRENTDEVICE.DeviceNo);
}

int adwin_get_par_block(int first, int count, int32_t dest[], int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Get_Par_Block(dest, first, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

void Get_Par_All(int32_t dest[])
{
    adwin_get_par_all(dest, CURRENTDEVICE.DeviceNo);
}

int adwin_get_par_all(int32_t dest[], int id)
{
    return adwin_get_par_block(1, 80, dest, id);
}

/*****************************************************************************/

void Set_FPar(int32_t index, float value)
{
    adwin_set_fpar(index, value, CURRENTDEVICE.DeviceNo);
}

int adwin_set_fpar(int index, float value, int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Set_FPar(index, value);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

void Set_FPar_Double(int32_t index, double value)
{
    adwin_set_fpar_double(index, value, CURRENTDEVICE.DeviceNo);
}

int adwin_set_fpar_double(int index, double value, int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Set_FPar_Double(index, value);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

float Get_FPar(int32_t index)
{
    return adwin_get_fpar(index, CURRENTDEVICE.DeviceNo);
}

float adwin_get_fpar(int index, int id)
{
    int err;
    float value = __adwin_get_fpar(index, &err, id);

    return (err == -1) ? err : value;
}

float  __adwin_get_fpar(int index, int *err, int id)
{
    adwin_dev_t dev;
    int fd;
    float dest = -1;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    dest = Generic_Driver_Get_FPar(index);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    if (err)
        *err = adwin_errno;

    return dest;
}

/*****************************************************************************/

double Get_FPar_Double(int32_t index)
{
    return adwin_get_fpar_double(index, CURRENTDEVICE.DeviceNo);
}

double adwin_get_fpar_double(int index, int id)
{
    int err;
    double value = __adwin_get_fpar_double(index, &err, id);

    return (err == -1) ? err : value;
}

double  __adwin_get_fpar_double(int index, int *err, int id)
{
    adwin_dev_t dev;
    int fd;
    double dest = -1;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    dest = Generic_Driver_Get_FPar_Double(index);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    if (err)
        *err = adwin_errno;

    return dest;
}

/*****************************************************************************/

void Get_FPar_Block(float dest[], int start_index, int count)
{
    adwin_get_fpar_block(start_index, count, dest, CURRENTDEVICE.DeviceNo);
}

int adwin_get_fpar_block(int start_index, int count, float dest[], int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Get_FPar_Block(dest, start_index, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

void Get_FPar_All(float dest[])
{
    adwin_get_fpar_all(dest, CURRENTDEVICE.DeviceNo);
}

int adwin_get_fpar_all(float dest[], int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Get_FPar_Block(dest, 1, 80);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

void Get_FPar_Block_Double(double dest[], int start_index, int count)
{
    adwin_get_fpar_block_double(start_index, count, dest, CURRENTDEVICE.DeviceNo);
}

int adwin_get_fpar_block_double(int start_index, int count, double dest[], int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Get_FPar_Block_Double(dest, start_index, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/
void Get_FPar_All_Double(double dest[])
{
    adwin_get_fpar_all_double(dest, CURRENTDEVICE.DeviceNo);
}

int adwin_get_fpar_all_double(double dest[], int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Get_FPar_Block_Double(dest, 1, 80);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

int32_t Data_Length(int32_t data_no)
{
    return adwin_data_length(data_no, CURRENTDEVICE.DeviceNo);
}

int adwin_data_length(int data_no, int id)
{
    adwin_dev_t dev;
    int fd;
    int32_t tmp;

    if ((fd = SETMUTEX(id, &dev)) < 0)
        return 0;

    tmp = Generic_Driver_Data_Length(data_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? 0 : tmp;
}

int32_t Clear_Data(int data_no)
{
    return adwin_clear_data(data_no, CURRENTDEVICE.DeviceNo);
}

int32_t Data_Type(int data_no)
{
    int tmp;
    tmp = Generic_Driver_GetDataType(data_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    return adwin_errno ? -1 : tmp;
}

int32_t _Set_Data(int32_t data_no, void *src, adwin_data_type_t adt, int32_t start_index, int32_t count, int device_no)
{
    adwin_dev_t dev;
    int fd;
    adwin_system_data_type_t asdt;
    void *ptr;

    if (adt == ADWIN_INTEGER)
        adt = ADWIN_LONG; // 4->3

    asdt = adwin_get_data_type(data_no, device_no);

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(device_no, &dev)) < 0) {
        return -1;
    }

    if (asdt == -1) {
        // adwin_get_data_type failed - it should have set
        // adwin_errno appropriately, so just exit
        CLEARMUTEX(&dev, fd);
        return -1;
    } else if ((asdt != ADWIN_SYSTEM_BYTE)
               && (asdt != ADWIN_SYSTEM_SHORT)
               && (asdt != ADWIN_SYSTEM_INTEGER)
               && (asdt != ADWIN_SYSTEM_LONG)
               && (asdt != ADWIN_SYSTEM_FLOAT)
               && (asdt != ADWIN_SYSTEM_DOUBLE)
               && (asdt != ADWIN_SYSTEM_INT64)) {
        // adwin_get_data_type returned some funny value,
        // which we map to k_Data_is_to_small here.
        adwin_errno = k_Data_is_to_small;
        CLEARMUTEX(&dev, fd);
        return -1;
    }

    ptr = Set_Data_convert(adt, src, asdt, count);
    if (!ptr ) {
        adwin_errno = -errno;
        CLEARMUTEX(&dev, fd);
        return -1;
    }

    Generic_Driver_Set_Data(data_no, ptr, adt, asdt, start_index, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    if (ptr != src) free(ptr);
    return adwin_errno ? -1 : 0;
}


int _Get_Data(int data_no, void *dest, adwin_data_type_t adt, int32_t start_index, int32_t count, int device_no)
{
    adwin_dev_t dev;
    int fd;
    adwin_system_data_type_t asdt;
    void *ptr;

    if (adt == ADWIN_INTEGER)
        adt = ADWIN_LONG; // 4->3

    asdt = adwin_get_data_type(data_no, device_no);

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(device_no, &dev)) < 0) {
        return -1;
    }

    if (asdt == -1) {
        // adwin_get_data_type failed - it should have set
        // adwin_errno appropriately, so just exit
        CLEARMUTEX(&dev, fd);
        return -1;
    } else if ((asdt != ADWIN_SYSTEM_BYTE)
               && (asdt != ADWIN_SYSTEM_SHORT)
               && (asdt != ADWIN_SYSTEM_INTEGER)
               && (asdt != ADWIN_SYSTEM_LONG)
               && (asdt != ADWIN_SYSTEM_INT64)
               && (asdt != ADWIN_SYSTEM_FLOAT)
               && (asdt != ADWIN_SYSTEM_DOUBLE)) {
        // adwin_get_data_type returned some funny value,
        // which we map to k_Data_is_to_small here.
        adwin_errno = k_Data_is_to_small;
        CLEARMUTEX(&dev, fd);
        return -1;
    }

    ptr = Get_Data_prepare_convert(asdt, dest, adt, count);
    if (!ptr) {
        adwin_errno = -errno;
        CLEARMUTEX(&dev, fd);
        return -1;
    }

    Generic_Driver_Get_Data(data_no, ptr, adt, asdt, start_index, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    Get_Data_do_convert(asdt, ptr, adt, dest, count);

    if (adt == ADWIN_SYSTEM_FLOAT)
        test_float_buffer_qnan(dest, count);
    if (adt == ADWIN_SYSTEM_DOUBLE)
        test_double_buffer_qnan(dest, count);

    if (ptr != dest) {
        //memcpy(dest, ptr, count * bytes_per_value(adt));
        free(ptr);
    }

    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/

void SetData_Long(int data_no, int32_t src[], int32_t start_index, int32_t count)
{
    _Set_Data(data_no, src, ADWIN_INTEGER, start_index, count, CURRENTDEVICE.DeviceNo);
}

/*****************************************************************************/

void GetData_Long(int data_no, int32_t dest[], int32_t start_index, int32_t count)
{
    _Get_Data(data_no, dest, ADWIN_INTEGER, start_index, count, CURRENTDEVICE.DeviceNo);
}

/*****************************************************************************/

void SetData_Float(int data_no, float src[], int32_t start_index, int32_t count)
{
    _Set_Data(data_no, src, ADWIN_FLOAT, start_index, count, CURRENTDEVICE.DeviceNo);
}

/*****************************************************************************/

void SetData_Double(int data_no, double src[], int32_t start_index, int32_t count)
{
    _Set_Data(data_no, src, ADWIN_DOUBLE, start_index, count, CURRENTDEVICE.DeviceNo);
}

/*****************************************************************************/

void GetData_Float(int data_no, float dest[], int32_t start_index, int32_t count)
{
    _Get_Data(data_no, dest, ADWIN_FLOAT, start_index, count, CURRENTDEVICE.DeviceNo);
}

/*****************************************************************************/

void GetData_Double(int data_no, double dest[], int32_t start_index, int32_t count)
{
    _Get_Data(data_no, dest, ADWIN_DOUBLE, start_index, count, CURRENTDEVICE.DeviceNo);
}

/*****************************************************************************/

void Data2File(const char *filename, int data_no, int32_t start_index, int32_t count, int mode)
{
    adwin_data2file(filename, data_no, start_index, count, mode, CURRENTDEVICE.DeviceNo);
}

int adwin_data2file(const char *filename, int data, int32_t first, int32_t cnt, int mode, int id)
{
    int fd;
    int32_t nbytes;
    char *ptr = NULL;

    switch (adwin_get_data_type(data, id)) {
        case 1:
            nbytes = cnt;
            ptr = (char *) malloc(nbytes);
            if (_Get_Data(data, ptr, ADWIN_BYTE, first, cnt, id)) {
                free(ptr);
                return -1;
            }
            break;
        case 2:
            nbytes = cnt * 2;
            ptr = (char *) malloc(nbytes);
            if (_Get_Data(data, ptr, ADWIN_SHORT, first, cnt, id)) {
                free(ptr);
                return -1;
            }
            break;
        case 3:
        case 4:
            nbytes = cnt * 4;
            ptr = (char *) malloc(nbytes);
            if (_Get_Data(data, ptr, ADWIN_LONG, first, cnt, id)) {
                free(ptr);
                return -1;
            }
            break;
        case 5:
            nbytes = cnt * 4;
            ptr = (char *) malloc(nbytes);
            if (_Get_Data(data, ptr, ADWIN_FLOAT, first, cnt, id)) {
                free(ptr);
                return -1;
            }
            break;
        case 6:
            nbytes = cnt * 8;
            ptr = (char *) malloc(nbytes);
            if (_Get_Data(data, ptr, ADWIN_DOUBLE, first, cnt, id)) {
                free(ptr);
                return -1;
            }
            break;
        case 7:
            nbytes = cnt * 8;
            ptr = (char *) malloc(nbytes);
            if (_Get_Data(data, ptr, ADWIN_INT64, first, cnt, id)) {
                free(ptr);
                return -1;
            }
            break;
        default:
            errno = ENOTSUP;
            return -1;
    }

    errno = 0;
    fd = open(filename, O_WRONLY | O_CREAT | (mode ? O_APPEND : O_TRUNC), 0644);
    if (fd < 0) {
        free(ptr);
        perror("open");
        return -1;
    }

    errno = 0;
    if (write(fd, ptr, nbytes) < nbytes) {
        perror ("write");
        free(ptr);
        return -1;
    }

    free(ptr);
    close(fd);

    return 0;
}

/*****************************************************************************/

void File2Data(const char *filename, adwin_data_type_t data_type, int data_no, int32_t start_index)
{
    adwin_file2data(filename, data_type, data_no, start_index, CURRENTDEVICE.DeviceNo);
}

int32_t adwin_file2data(const char *filename, adwin_data_type_t data_type, int data_no, int32_t start_index, int id)
{
    FILE *streams = NULL;
    int ret;

    streams = fopen(filename, "rb" );
    if (streams != NULL) {
        int32_t filesize, *data = NULL;

        fseek(streams, 0, SEEK_END);
        filesize = ftell(streams);
        fseek(streams, 0, SEEK_SET);

        data = (int32_t *)malloc(filesize);
        if (! data) {
            fclose(streams);
            return -1;
        }
        ret = fread(data, 1, filesize, streams);
        fclose(streams);
        if (ret != filesize) {
            adwin_errno = k_File_not_valid;
            return adwin_errno ? -1 : 0;
        }
        _Set_Data(data_no, data, data_type, start_index, filesize / 4, id);
        free(data);
        return adwin_errno ? -1 : 0;
    } else {
        adwin_errno = k_File_not_valid;
        return adwin_errno ? -1 : 0;
    }
    return 0;
}

/*****************************************************************************/

int32_t Fifo_Empty(int fifo_no)
{
    return adwin_fifo_empty(fifo_no, CURRENTDEVICE.DeviceNo);
}

int adwin_fifo_empty(int fifo_no, int id)
{
    int err, cnt;
    cnt = __adwin_fifo_empty(fifo_no, &err, id);

    return ((err == -1) ? err : cnt);
}

int __adwin_fifo_empty(int fifo_no, int *err, int id)
{
    adwin_dev_t dev;
    int fd;
    int dest = 0;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    dest = Generic_Driver_Fifo_Empty(fifo_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    if (err)
        *err = adwin_errno;
    return adwin_errno ? -1 : dest;
}

/*****************************************************************************/

int32_t Fifo_Full(int fifo_no)
{
    return adwin_fifo_full(fifo_no, CURRENTDEVICE.DeviceNo);
}

int adwin_fifo_full(int fifo_no, int id)
{
    int cnt, err;
    cnt = __adwin_fifo_full(fifo_no, &err, id);
    return (err == -1) ? err : cnt;
}

// todo: error handling

int __adwin_fifo_full(int fifo_no, int *err, int id)
{
    adwin_dev_t dev;
    int fd;
    int dest = 0;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    dest = Generic_Driver_Fifo_Full(fifo_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    if (err)
        *err = adwin_errno;
    return adwin_errno ? -1 : dest;
}

/*****************************************************************************/

void Fifo_Clear(int fifo_no)
{
    adwin_fifo_clear(fifo_no, CURRENTDEVICE.DeviceNo);
}

int adwin_fifo_clear(int fifo_no, int id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0)
        return -1;

    Generic_Driver_Fifo_Clear(fifo_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/
int _Set_Fifo(int32_t fifo_no, void *src, adwin_data_type_t adt, int32_t count, int32_t device_no)
{
    adwin_dev_t dev;
    int fd;
    adwin_system_data_type_t asdt;
    void *ptr;

    asdt = adwin_get_data_type(fifo_no, device_no);

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(device_no, &dev)) < 0) {
        return -1;
    }

    if (asdt == -1) {
        // adwin_get_data_type failed - it should have set
        // adwin_errno appropriately, so just exit
        CLEARMUTEX(&dev, fd);
        return -1;
    } else if ((asdt != ADWIN_SYSTEM_BYTE)
               && (asdt != ADWIN_SYSTEM_SHORT)
               && (asdt != ADWIN_SYSTEM_INTEGER)
               && (asdt != ADWIN_SYSTEM_LONG)
               && (asdt != ADWIN_SYSTEM_FLOAT)
               && (asdt != ADWIN_SYSTEM_DOUBLE)
               && (asdt != ADWIN_SYSTEM_INT64)) {
        // adwin_get_data_type returned some funny value,
        // which we map to k_Data_is_to_small here.
        adwin_errno = k_Data_is_to_small;
        CLEARMUTEX(&dev, fd);
        return -1;
    }

    ptr = Set_Data_convert(adt, src, asdt, count);

    if (!ptr ) {
        adwin_errno = -errno;
        CLEARMUTEX(&dev, fd);
        return -1;
    }

    Generic_Driver_Set_Fifo(fifo_no, asdt, ptr, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    if (ptr != src) free(ptr);

    return adwin_errno ? -1 : 0;
}


int _Get_Fifo(int fifo_no, void *dest, adwin_data_type_t adt, int32_t count, int device_no)
{
    adwin_dev_t dev;
    int fd;
    adwin_system_data_type_t asdt;
    void *ptr;

    asdt = adwin_get_data_type(fifo_no, device_no);

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(device_no, &dev)) < 0) {
        return -1;
    }

    if (asdt == -1) {
        // adwin_get_data_type failed - it should have set
        // adwin_errno appropriately, so just exit
        CLEARMUTEX(&dev, fd);
        return -1;
    } else if ((asdt != ADWIN_SYSTEM_BYTE)
               && (asdt != ADWIN_SYSTEM_SHORT)
               && (asdt != ADWIN_SYSTEM_INTEGER)
               && (asdt != ADWIN_SYSTEM_LONG)
               && (asdt != ADWIN_SYSTEM_FLOAT)
               && (asdt != ADWIN_SYSTEM_DOUBLE)
               && (asdt != ADWIN_SYSTEM_INT64)) {
        // adwin_get_data_type returned some funny value,
        // which we map to k_Data_is_to_small here.
        adwin_errno = k_Data_is_to_small;
        CLEARMUTEX(&dev, fd);
        return -1;
    }

    ptr = Get_Data_prepare_convert(asdt, dest, adt, count);

    if (!ptr ) {
        adwin_errno = -errno;
        CLEARMUTEX(&dev, fd);
        return -1;
    }

    Generic_Driver_Get_Fifo(fifo_no, asdt, ptr, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);

    Get_Data_do_convert(asdt, ptr, adt, dest, count);
    if (adt == ADWIN_SYSTEM_FLOAT)
        test_float_buffer_qnan(dest, count);
    if (adt == ADWIN_SYSTEM_DOUBLE)
        test_double_buffer_qnan(dest, count);

    if (ptr != dest) free(ptr);

    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/
void SetFifo_Long(int fifo_no, int32_t src[], int32_t count)
{
    _Set_Fifo(fifo_no, src, ADWIN_LONG, count, CURRENTDEVICE.DeviceNo);
}
/*****************************************************************************/
void GetFifo_Long(int fifo_no, int32_t dest[], int32_t count)
{
    _Get_Fifo(fifo_no, dest, ADWIN_LONG, count, CURRENTDEVICE.DeviceNo);
}
/*****************************************************************************/
void SetFifo_Float(int fifo_no, float src[], int32_t count)
{
    _Set_Fifo(fifo_no, src, ADWIN_FLOAT, count, CURRENTDEVICE.DeviceNo);
}
/*****************************************************************************/
void GetFifo_Float(int fifo_no, float dest[], int32_t count)
{
    _Get_Fifo(fifo_no, dest, ADWIN_FLOAT, count, CURRENTDEVICE.DeviceNo);
}
/*****************************************************************************/
void SetFifo_Double(int fifo_no, double src[], int32_t count)
{
    _Set_Fifo(fifo_no, src, ADWIN_DOUBLE, count, CURRENTDEVICE.DeviceNo);
}
/*****************************************************************************/
void GetFifo_Double(int fifo_no, double dest[], int32_t count)
{
    _Get_Fifo(fifo_no, dest, ADWIN_DOUBLE, count, CURRENTDEVICE.DeviceNo);
}
/*****************************************************************************/
void SetData_String(int data_no, char *src)
{
    adwin_set_data_string(data_no, src, CURRENTDEVICE.DeviceNo);
}

int adwin_set_data_string(int data_no, const char *src, int id)
{
    size_t len;
    adwin_dev_t dev;
    int fd;

    len = strlen(src);

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0) {
        return -1;
    }

    Generic_Driver_SetData_String(data_no, src);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? -1 : 0;
}
/*****************************************************************************/

int32_t GetData_String(int32_t data_no, char *dest, int32_t max_count)
{
    return adwin_get_data_string(data_no, max_count, dest, CURRENTDEVICE.DeviceNo);
}

int32_t adwin_get_data_string(int32_t data_no, int32_t count, char *dest, int32_t id)
{
    adwin_dev_t dev;
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0) {
        return -1;
    }

    Generic_Driver_GetData_String(data_no, dest, count);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? -1 : 0;
}

/*****************************************************************************/
int32_t String_Length(int32_t data_no)
{
    return adwin_strlen(data_no, CURRENTDEVICE.DeviceNo);
}

int32_t adwin_strlen(int data_no, int id)
{
    int32_t len;
    int fd;
    adwin_dev_t dev;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(id, &dev)) < 0) {
        return -1;
    }

    len = Generic_Driver_String_Length(data_no);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? -1 : len;
}

/*****************************************************************************/
int32_t Get_Last_Error()
{
    int32_t temp = adwin_errno;
    adwin_errno = 0;
    return temp;
}

char *Get_Last_Error_Text(int32_t Errno)
{
    return (char *)(adwin_strerror(Errno));
}
/*****************************************************************************/

int16_t Get_Known_DeviceNo(int32_t devices[], int32_t *count_devices)
{
    int i;
    PRINTDEBUG(DBG_LIBADWIN,
               ("Get_Known_DeviceNo called with %p %d\n", devices, *count_devices));

    // check if the given array is large enough
    if (*count_devices < getDeviceCount()) {
        // if not, return required size
        *count_devices = getDeviceCount();
        return 1;
    }

    // copy device numbers to array
    for (i = 0; i < getDeviceCount(); i++) {
        adwin_dev_t tmp = getDevice(i);
        devices[i] = (short)tmp.dev_name;
    }

    // let the caller know how much we copied
    *count_devices = getDeviceCount();

    return 0;
}

/*****************************************************************************/

const char *Get_Library_Version()
{
    return VersionString;
}

char *Get_Password()
{
    adwin_dev_t dev;
    char *buf = calloc(1, PWLEN + 1);
    int fd;

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(CURRENTDEVICE.DeviceNo, &dev)) < 0)
        return NULL;

    CLEARMUTEX(&dev, fd);
    strncpy(buf, dev.dev_net_host_password, PWLEN);

    return buf;
}


int32_t adwin_set_value(int32_t addr, void *value, int32_t value_size)
{
    adwin_dev_t dev;
    int fd;

    //printf("adwin_set_value addr=0x%x, value=%d, size=%d\n", addr, (int) *(int*)value, value_size);

    if ((fd = SETMUTEX_INIT_GEN_DRIVER(CURRENTDEVICE.DeviceNo, &dev)) < 0) {
        return -1;
    }

    Generic_Driver_Set_Value(addr, value, value_size);
    adwin_errno = Generic_Driver_Get_Last_Error();
    CLEARMUTEX(&dev, fd);
    return adwin_errno ? -1 : 0;
}
